// BasicShader.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <GL/glew.h>
#include <GL/glut.h>
#include <SDL/SDL.h> //NEW
#include <SDL/SDL_image.h> // NEW
#include <iostream>
#include <string.h>
#include <fstream>
#include <sstream>

#define SDL_SUBSYSTEMS SDL_INIT_VIDEO

GLuint programObject;
GLfloat rotation=0;
GLuint g_texture = 0;			// ID textury
GLUquadricObj *g_Quadric;

GLuint CreateTexture(const char* filename);	// Vytvori texturu
GLuint CreateTexture(SDL_Surface *surface);
bool SwapSurfaceRows(SDL_Surface *surface);	// Prohodi radky v obrazku


bool LoadTXTFile(const char *fileName, std::string &textFile)
	{
		std::stringstream strStream;

		std::ifstream file(fileName, std::ifstream::in);
		if(!file.good())
			return false;

		strStream << file.rdbuf();
		textFile = strStream.str();
		file.close();

		return true;
	}

/*
 * Vytvori OpenGL texturu z obrazku ulozeneho na disku;
 * vrati nulu, pokud soubor neexistuje nebo pokud neco selze
 */

GLuint CreateTexture(const char* filename)
{
	GLuint texture;
	SDL_Surface *img;

	img = IMG_Load(filename);// Nazapomenout prilinkovat SDL_image

	if(img == NULL)
	{
		fprintf(stderr, "Unable to load '%s': %s\n",
				filename, SDL_GetError());
		return 0;
	}

	texture = CreateTexture(img);
	SDL_FreeSurface(img);

	return texture;
}


/*
 * Vytvori OpenGL texturu ze SDL_Surface, v pripade neuspechu vrati nulu
 */

GLuint CreateTexture(SDL_Surface *surface)
{
	GLuint texture;		// Vysledna textura
	Uint32 saved_flags;	// Pomocne pro ulozeni flagu
	Uint8  saved_alpha;
	SDL_Surface *tmp_img;	// Obrazek pro vytvoreni textury

	// Vytvori prazdny RGB surface
	tmp_img = SDL_CreateRGBSurface(
			SDL_SWSURFACE,		// Softwarovy surface
			surface->w, surface->h,	// Sirka, vyska
			32,			// Barevna hloubka
#if SDL_BYTEORDER == SDL_LIL_ENDIAN
			0x000000FF,		// OpenGL RGBA maska
			0x0000FF00,
			0x00FF0000,
			0xFF000000
#else
			0xFF000000,
			0x00FF0000,
			0x0000FF00,
			0x000000FF
#endif
			);

	if(tmp_img == NULL)
		return 0;

	// Ulozi atributy alfa blendingu
	saved_flags = surface->flags & (SDL_SRCALPHA|SDL_RLEACCELOK);
	saved_alpha = surface->format->alpha;

	if((saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA)
		SDL_SetAlpha(surface, 0, 0);

	// Zkopiruje surface do obrazku pro texturu
	SDL_Rect area;
	area.x = 0;
	area.y = 0;
	area.w = surface->w;
	area.h = surface->h;
	SDL_BlitSurface(surface, &area, tmp_img, &area);

	// Obnovi atributy alfa blendingu
	if((saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA)
		SDL_SetAlpha(surface, saved_flags, saved_alpha);

	// Prohodi radky
	if(!SwapSurfaceRows(tmp_img))
	{
		SDL_FreeSurface(tmp_img);
		return 0;
	}

	// Vytvori texturu
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D,
		     0,
		     GL_RGBA,
		     surface->w, surface->h,
		     0,
		     GL_RGBA,
		     GL_UNSIGNED_BYTE,
		     tmp_img->pixels);

	SDL_FreeSurface(tmp_img);
	return texture;
}


/*
 * Prohodi radky v obrazku (prvni za posledni, druhy za predposledni atd.),
 */

bool SwapSurfaceRows(SDL_Surface *surface)
{
	// Ukazatele na prohazovane radky
	Uint8 *hi_row, *lo_row, *tmp_row;

	// Alokace pameti pro pomocny radek
	tmp_row = (Uint8 *)malloc(surface->pitch);

	if(tmp_row == NULL)
	{
		fprintf(stderr, "Unable to allocate memory.\n");
		return false;
	}

	// Ukazatele na prvni a posledni radek
	hi_row = (Uint8 *)surface->pixels;
	lo_row = hi_row + (surface->h * surface->pitch) - surface->pitch;

	if(SDL_MUSTLOCK(surface))
	{
		if(SDL_LockSurface(surface) == -1)
		{
			fprintf(stderr, "Unable to lock surface.\n");
			free(tmp_row);
			return false;
		}
	}

	for(int i = 0; i < surface->h / 2; i++)
	{
		memcpy(tmp_row, hi_row, surface->pitch);
		memcpy(hi_row, lo_row, surface->pitch);
		memcpy(lo_row, tmp_row, surface->pitch);

		hi_row += surface->pitch;// Dalsi/predchozi radek
		lo_row -= surface->pitch;
	}

	if (SDL_MUSTLOCK(surface))
	{
		SDL_UnlockSurface(surface);
	}

	free(tmp_row);// Uvolni pomocnou pamet

	return true;
}

void InitGlut(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(500, 500);
	glutInitWindowPosition(100, 100);
	glutCreateWindow(argv[0]);
}

void GLInit()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_FLAT);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D); //NEW

	//NEW
	g_Quadric= gluNewQuadric();
	gluQuadricDrawStyle(g_Quadric, GLU_FILL);
	gluQuadricOrientation(g_Quadric, GLU_OUTSIDE);
	gluQuadricNormals(g_Quadric, GLU_FLAT);
	gluQuadricTexture(g_Quadric, GLU_TRUE);
}

bool SDLInit()
{
	if(SDL_Init(SDL_SUBSYSTEMS) == -1)	// Inicializace SDL
	{
		fprintf(stderr, "Unable to initialize SDL: %s\n",
				SDL_GetError());
		return false;
	}
	// Nahraje obrazek a vytvori texturu
	if((g_texture = CreateTexture("./SDLlogo.png")) == 0)
		return false;
	return true;
}

void display(void)
{
	float c1[]={1.0, 0.0, 0.0, 1.0};
	float c2[]={0.0, 1.0, 0.0, 1.0};
	float c3[]={0.0, 0.0, 1.0, 1.0};

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	gluLookAt(0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

	glRotatef(rotation, 1.0, 0.0, 0.0);
	//glutSolidTeapot(1.0);
	gluSphere(g_Quadric, 0.75, 15, 10);

	glFlush();
	glutSwapBuffers();
}

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	        //(Left, Right, Bottom, Top, Near, Far);
	glFrustum(-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
	glMatrixMode(GL_MODELVIEW);
}

void keyboardSpec(int key, int x, int y)
{
	switch (key){
		case GLUT_KEY_UP:
			rotation+=1.0;
			glutPostRedisplay();
			break;
		case GLUT_KEY_DOWN:
			rotation-=1.0;
			glutPostRedisplay();
			break;
		default:
			break;
	}
}

void keyboard(unsigned char key, int x, int y)
{if ((int)key == 27) exit(0);}

void GlutCallbacks()
{
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutSpecialFunc(keyboardSpec);
	glutKeyboardFunc(keyboard);
}

void CheckVersions()
{
	char *version, *extensions, *renderer, *shadingLanguage, *vendor;

	//Dostaneme retezec obsahujici dostupne extenze
    extensions = (char*)glGetString(GL_EXTENSIONS);
	version = (char*)glGetString(GL_VERSION);
	renderer = (char*)glGetString(GL_RENDERER);
	shadingLanguage = (char*)glGetString(GL_SHADING_LANGUAGE_VERSION);
	vendor = (char*)glGetString(GL_VENDOR);

	std::cout << "Vrobce: ";
	std::cout << vendor << "\n";
	std::cout << "Grafick karta: ";
	std::cout << renderer << "\n";
	std::cout << "Verze OpenGL: ";
	std::cout << version << "\n";
	std::cout << "Verze GLSL: ";
	std::cout << shadingLanguage << "\n";
	//std::cout << "Extenze: ";
	//std::cout << extensions;
}

void CreateShader()
{
	GLint status;
	  
	// 1 Vytvoreni objektu shaderu
	GLuint vertexShaderObject;
	GLuint fragmentShaderObject;
	vertexShaderObject= glCreateShader(GL_VERTEX_SHADER);
	fragmentShaderObject= glCreateShader(GL_FRAGMENT_SHADER);
	  
	// 2 Nahrati zdrojoveho kodu ze souboru VertexShader.sh
	std::string vertexShaderTextString;
	std::string fragmentShaderTextString;
	if(!LoadTXTFile("VertexShader5.sh", vertexShaderTextString))
		std::cout << "\nFile VertexShader5.sh not loaded, ERROR\n";
	if(!LoadTXTFile("FragmentShader5.sh", fragmentShaderTextString))
		std::cout << "\nFile FragmentShader5.sh not loaded, ERROR\n";
	const char * vs= vertexShaderTextString.c_str();
	const char * fs= fragmentShaderTextString.c_str();
	glShaderSource(vertexShaderObject, 1, &vs, NULL);
	glShaderSource(fragmentShaderObject, 1, &fs, NULL);
	  
	// 3 Kompilace shaderu
	glCompileShader(vertexShaderObject);
	glGetShaderiv(vertexShaderObject, GL_COMPILE_STATUS, &status);
	if(status) std::cout << "Vertex shader compilation OK\n";
	else
	{
		std::cout << "\nVertex shader compilation ERROR\n";
		GLchar infoLog[5000];
		glGetShaderInfoLog(vertexShaderObject, 5000, NULL, infoLog);
		std::cout << infoLog <<"\n";
	}
	glCompileShader(fragmentShaderObject);
	glGetShaderiv(fragmentShaderObject, GL_COMPILE_STATUS, &status);
	if(status) std::cout << "Fragment shader compilation OK\n";
	else
	{
		std::cout << "\nFragment shader compilation ERROR\n";
		GLchar infoLog[5000];
		glGetShaderInfoLog(fragmentShaderObject, 5000, NULL, infoLog);
		std::cout << infoLog <<"\n";
	}
	  
	// 4 Vytvoreni programoveho objektu
	GLuint programObject;
	programObject= glCreateProgram();
	  
	// 5 Pripojeni objektu shaderu k programovemu objektu
	glAttachShader(programObject, vertexShaderObject);
	glAttachShader(programObject, fragmentShaderObject);
	  
	// 6 Slinkovani programu
	glLinkProgram(programObject);
	  
	// 7 Pouziti programu
	glUseProgram(programObject);

	// zjisteni lokace uniformnich promennych
	
	GLint LightPositionLoc;
	
	LightPositionLoc= glGetUniformLocation(programObject, "LightPosition");
	
	glUniform3f(LightPositionLoc, 0.0f, 4.0f, 0.0f);
		
}

int _tmain(int argc, char** argv)
{
	InitGlut(argc, argv);
	SDLInit();
	GLInit();
	glewInit();

	CheckVersions();
	GlutCallbacks();

	//CreateShader();

	glutMainLoop();
	return 0;
}
